-- ACE declarations

shardacecata = LibStub("AceAddon-3.0"):NewAddon("shardacecata", "AceEvent-3.0", "AceConsole-3.0")

local BD = LibStub("ButtonDaemon-1.0")
local AceDB = LibStub("AceDB-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local dialog = LibStub("AceConfigDialog-3.0")


-- Option tabs and handlers

local options = {
	type = 'group', childGroups = "tab",
	args = {
		main = {
			type = 'group', name ='Main Buttons', order = 1, 
			args = {
				radius = {
					type = 'range', name = "Radius",
					desc = "Set the Radius for placing the main buttons",
					handler = shardacecata,
					min = 15, max = 1000,
					step = 1,
					--pattern = "%d",
					get = "getRadius",
					set = "setRadius",
					order = 1,
				},
				angle = {
					type = 'range', name = "Start Angle",
					desc = "Set the starting Angle for placing the main buttons",
					handler = shardacecata,
					min = 0, max = 359,
					step = 1,
					--pattern = "%d",
					get = "getAngle",
					set = "setAngle",
					order = 2,
				},
				override = {
					type = 'toggle', name = "Custom Increment",
					desc = "Toggle over-riding increment angle",
					handler = shardacecata,
					get = "getOverrideState",
					set = "setOverrideState",
					order = 3,
				},
				increment = {
					type = 'range', name = "Increment Angle",
					desc = "Set the Increment Angle for placing the main buttons",
					handler = shardacecata,
					min = -359, max = 359,
					step = 1,
					--pattern = "%d",
					get = "getIncrement",
					set = "setIncrement",
					order = 4,
				},
				reverse = {
					type = 'toggle', name = "Reverse Placement",
					desc = "Toggle direction of button placement. Does not apply to custom Increment Angles",
					handler = shardacecata,
					get = "getReverseState",
					set = "setReverseState",
					order = 5,
				},
				anchor = {
					type = 'input', name = "Anchor Frame",
					desc = "Set the Frame to place the main buttons around. Defaults to 'Minimap'.  Can also use 'cursor ' for an option.  ONLY THESE TWO OPTIONS WORK, YOU WILL GET AN ERROR OTHERWISE.  (hopefully I can fix this sometime in the future.)",
					handler = shardacecata,
					--pattern = "%d",
					get = "getAnchor",
					set = "setAnchor",
					order = 6,
				},
				straight = {
					type = 'toggle', name = "Toggle Alignment",
					desc = "Toggle alignment of main buttons between circular and straight",
					handler = shardacecata,
					get = "getStraightState",
					set = "setStraightState",
					order = 7,
				},
				offx = {
					type = 'range', name = "X Offset",
					desc = "Set the X Offset for Placing the main buttons",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "getMainOffX",
					set = "setMainOffX",
					order = 8,
				},
				offy = {
					type = 'range', name = "Y Offset",
					desc = "Set the Y Offset for Placing the main buttons",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "getMainOffY",
					set = "setMainOffY",
					order = 9,
				},
			},
		},
		firstrow = {
			type = 'group', name ='1st Row', order = 2,
			args = {
				initx = {
					type = 'range', name = "Initial X",
					desc = "Set the Initial X Posistion for Placing the first row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get1InitX",
					set = "set1InitX",
					order = 1,
				},
				inity = {
					type = 'range', name = "Initial Y",
					desc = "Set the Initial Y Posistion for Placing the first row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get1InitY",
					set = "set1InitY",
					order = 2,
				},
				offx = {
					type = 'range', name = "X Offset",
					desc = "Set the X Offset for Placing the first row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get1OffX",
					set = "set1OffX",
					order = 3,
				},
				offy = {
					type = 'range', name = "Y Offset",
					desc = "Set the Y Offset for Placing the first row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get1OffY",
					set = "set1OffY",
					order = 4,
				},
			},
		},
		secondrow = {
			type = 'group', name ='2nd Row', order = 3,
			args = {
				initx = {
					type = 'range', name = "Initial X",
					desc = "Set the Initial X Posistion for Placing the second row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get2InitX",
					set = "set2InitX",
					order = 1,
				},
				inity = {
					type = 'range', name = "Initial Y",
					desc = "Set the Initial Y Posistion for Placing the second row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get2InitY",
					set = "set2InitY",
					order = 2,
				},
				offx = {
					type = 'range', name = "X Offset",
					desc = "Set the X Offset for Placing the second row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get2OffX",
					set = "set2OffX",
					order = 3,
				},
				offy = {
					type = 'range', name = "Y Offset",
					desc = "Set the Y Offset for Placing the second row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get2OffY",
					set = "set2OffY",
					order = 4,
				},
			},
		},
		thirdrow = {
			type = 'group', name ='3rd Row', order = 4,
			args = {
				initx = {
					type = 'range', name = "Initial X",
					desc = "Set the Initial X Posistion for Placing the third row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get3InitX",
					set = "set3InitX",
					order = 1,
				},
				inity = {
					type = 'range', name = "Initial Y",
					desc = "Set the Initial Y Posistion for Placing the third row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get3InitY",
					set = "set3InitY",
					order = 2,
				},
				offx = {
					type = 'range', name = "X Offset",
					desc = "Set the X Offset for Placing the third row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get3OffX",
					set = "set3OffX",
					order = 3,
				},
				offy = {
					type = 'range', name = "Y Offset",
					desc = "Set the Y Offset for Placing the third row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get3OffY",
					set = "set3OffY",
					order = 4,
				},
			},
		},
		fourthrow = {
			type = 'group', name ='4th Row', order = 5,
			args = {
				initx = {
					type = 'range', name = "Initial X",
					desc = "Set the Initial X Posistion for Placing the forth row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get4InitX",
					set = "set4InitX",
					order = 1,
				},
				inity = {
					type = 'range', name = "Initial Y",
					desc = "Set the Initial Y Posistion for Placing the forth row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get4InitY",
					set = "set4InitY",
					order = 2,
				},
				offx = {
					type = 'range', name = "X Offset",
					desc = "Set the X Offset for Placing the forth row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get4OffX",
					set = "set4OffX",
					order = 3,
				},
				offy = {
					type = 'range', name = "Y Offset",
					desc = "Set the Y Offset for Placing the forth row",
					handler = shardacecata,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get4OffY",
					set = "set4OffY",
					order = 4,
				},
			},
		},	
		miscoptions = {
			type = 'group', name ='Miscellaneous', order = 9, 
			args = {
				showtooltips = {
					type = 'toggle', name = "Show tool-tips",
					desc = "Show tool-tips of spells associated with Shard Ace buttons",
					handler = shardacecata,
					get = "getShowTooltips",
					set = "setShowTooltips",
					order = 1,
				},
				soundstate = {
					type = 'toggle', name = "Play soulstone sound",
					desc = "Play a sound file upon expiry of your applied soulstone. The file name is soulstone.ogg and you can replace this with your own file.",
					handler = shardacecata,
					get = "getSoundState",
					set = "setSoundState",
					order = 2,
				},
			   },
		},
	}
}

function shardacecata:getMainOffX()
	return self.db.profile.mainoffx
end
function shardacecata:setMainOffX(info, s)
	self.db.profile.mainoffx = s
	self:PositionButtons()
end
function shardacecata:getMainOffY()
	return self.db.profile.mainoffy
end
function shardacecata:setMainOffY(info, s)
	self.db.profile.mainoffy = s
	self:PositionButtons()
end
function shardacecata:getStraightState()
	return self.db.profile.straight
end
function shardacecata:setStraightState()
	self.db.profile.straight = not self.db.profile.straight
	self:PositionButtons()
end
function shardacecata:getRadius()
	return self.db.profile.radius
end
function shardacecata:setRadius(info, s)
	self.db.profile.radius = s
	self:PositionButtons()
end
function shardacecata:getAngle()
	return self.db.profile.angle
end
function shardacecata:setAngle(info, s)
	self.db.profile.angle = s
	self:PositionButtons()
end
function shardacecata:getReverseState()
	if self.db.profile.reverse == -1 then
		return true
	else
		return false
	end
end
function shardacecata:setReverseState()
	self.db.profile.reverse = self.db.profile.reverse * -1
	self:PositionButtons()
end
function shardacecata:getOverrideState()
	return self.db.profile.override
end
function shardacecata:setOverrideState()
	self.db.profile.override = not self.db.profile.override
	self:PositionButtons()
end
function shardacecata:getIncrement()
	return self.db.profile.increment
end
function shardacecata:setIncrement(info, s)
	self.db.profile.increment = s
	self:PositionButtons()
end
function shardacecata:getAnchor()
	return self.db.profile.anchor
end
function shardacecata:setAnchor(info, s)
	self.db.profile.anchor = s
	if strlower(s) == "cursor" then
		self.db.profile.curx, self.db.profile.cury = GetCursorPosition()
		self.anchor:SetPoint("CENTER",UIParent,"BOTTOMLEFT",self.db.profile.curx / UIParent:GetEffectiveScale(), self.db.profile.cury / UIParent:GetEffectiveScale() )
	else
		self.anchor:SetPoint("CENTER",UIParent,"BOTTOMLEFT",_G[s]:GetCenter() )
	end
	self:PositionButtons()
end

function shardacecata:get1InitX()
	return self.db.profile.oneinitx
end
function shardacecata:set1InitX(info, s)
	self.db.profile.oneinitx = s
	self:PositionButtons()
end
function shardacecata:get1InitY()
	return self.db.profile.oneinity
end
function shardacecata:set1InitY(info, s)
	self.db.profile.oneinity = s
	self:PositionButtons()
end
function shardacecata:get1OffX()
	return self.db.profile.oneoffx
end
function shardacecata:set1OffX(info, s)
	self.db.profile.oneoffx = s
	self:PositionButtons()
end
function shardacecata:get1OffY()
	return self.db.profile.oneoffy
end
function shardacecata:set1OffY(info, s)
	self.db.profile.oneoffy = s
	self:PositionButtons()
end

function shardacecata:get2InitX()
	return self.db.profile.twoinitx
end
function shardacecata:set2InitX(info, s)
	self.db.profile.twoinitx = s
	self:PositionButtons()
end
function shardacecata:get2InitY()
	return self.db.profile.twoinity
end
function shardacecata:set2InitY(info, s)
	self.db.profile.twoinity = s
	self:PositionButtons()
end
function shardacecata:get2OffX()
	return self.db.profile.twooffx
end
function shardacecata:set2OffX(info, s)
	self.db.profile.twooffx = s
	self:PositionButtons()
end
function shardacecata:get2OffY()
	return self.db.profile.twooffy
end
function shardacecata:set2OffY(info, s)
	self.db.profile.twooffy = s
	self:PositionButtons()
end

function shardacecata:get3InitX()
	return self.db.profile.threeinitx
end
function shardacecata:set3InitX(info, s)
	self.db.profile.threeinitx = s
	self:PositionButtons()
end
function shardacecata:get3InitY()
	return self.db.profile.threeinity
end
function shardacecata:set3InitY(info, s)
	self.db.profile.threeinity = s
	self:PositionButtons()
end
function shardacecata:get3OffX()
	return self.db.profile.threeoffx
end
function shardacecata:set3OffX(info, s)
	self.db.profile.threeoffx = s
	self:PositionButtons()
end
function shardacecata:get3OffY()
	return self.db.profile.threeoffy
end
function shardacecata:set3OffY(info, s)
	self.db.profile.threeoffy = s
	self:PositionButtons()
end

function shardacecata:get4InitX()
	return self.db.profile.fourinitx
end
function shardacecata:set4InitX(info, s)
	self.db.profile.fourinitx = s
	self:PositionButtons()
end
function shardacecata:get4InitY()
	return self.db.profile.fourinity
end
function shardacecata:set4InitY(info, s)
	self.db.profile.fourinity = s
	self:PositionButtons()
end
function shardacecata:get4OffX()
	return self.db.profile.fouroffx
end
function shardacecata:set4OffX(info, s)
	self.db.profile.fouroffx = s
	self:PositionButtons()
end
function shardacecata:get4OffY()
	return self.db.profile.fouroffy
end
function shardacecata:set4OffY(info, s)
	self.db.profile.fouroffy = s
	self:PositionButtons()
end
function shardacecata:getReverseSort()
	return self.db.profile.reversesort
end
function shardacecata:setReverseSort()
	self.db.profile.reversesort = not self.db.profile.reversesort
end
function shardacecata:getShowTooltips()
  return self.db.profile.showtooltips
end
function shardacecata:setShowTooltips()
  self.db.profile.showtooltips = not self.db.profile.showtooltips
  self:SetupTooltips()
end
function shardacecata:getSoundState()
  return self.db.profile.soundstate
end
function shardacecata:setSoundState()
  self.db.profile.soundstate = not self.db.profile.soundstate
end
function shardacecata:getSayMessages()
 return self.db.profile.saymessages
end
function shardacecata:setSayMessages()
 self.db.profile.saymessages = not self.db.profile.saymessages
end


-- Item and Spell IDs

shardacecata.SoulShardID = 6265
shardacecata.HealthstoneID = {
  [5512] = 1,
  [19004] = 1,
  [19005] = 1,

  [5511] = 1,
  [19006] = 1,
  [19007] = 1,

  [5509] = 1,
  [19008] = 1,
  [19009] = 1,

  [5510] = 1,
  [19010] = 1,
  [19011] = 1,

  [9421] = 1,
  [19012] = 1,
  [19013] = 1,

  [22103] = 1,
  [22104] = 1,
  [22105] = 1,

  [36889] = 1,
  [36890] = 1,
  [36891] = 1,

  [36892] = 1,
  [36893] = 1,
  [36894] = 1,
}
shardacecata.SoulstoneID = {
  [6203] = 1,
  [5232] = 1,
  [16892] = 1,
  [16893] = 1,
  [16895] = 1,
  [16896] = 1,
  [22116] = 1,
  [36895] = 1,
}
shardacecata.spellID = {
	CreateSS = 20707,
	CreateHS = 6201,
	
	SoulHarvest = 79268,
	Enslave = 1098,
	SoulLink = 19028,
	Empower = 47193,
	Soulburn = 74434,
	Imp = 688,
	Voidwalker = 697,
	Succubus = 366222,
	Felhunter = 691,
	Infernal = 1122,
	--Infernal = 112921,
	Doomguard = 18540,
	--Doomguard = 112927,
	Felguard = 30146,
	
	DarkIntent = 109773,
	UnendingBreath = 5697,
	DemonSkin = 20798,
	DemonArmor = 687,
	FelArmor = 28176,
	ShadowWard = 6229,
	Metamorph = 47241,
	
	Felsteed = 5784,
	Dreadsteed = 23161,
	Summon = 698,
	Eye = 126,
	Banish = 710,
	SoulWell = 29893,
	SummonCircle = 48018,
	PortCircle = 48020,	
}


-- Other local variables

local shardCount, lastSort

local CreateButtonsFlag, SetupButtonsFlag, PositionButtonsFlag

local SoulstoneTarget


-- Subroutines

function shardacecata:displayCurrentShardCount()
  local shards = UnitPower("player", 7)

  if not shardCount then
    shardCount = self.buttons[1]:CreateFontString("DDShardCount", "OVERLAY")
    shardCount:SetFontObject(GameFontHighlightSmall)
    shardCount:SetTextColor(1,1,1,1)
    shardCount:SetAllPoints()
  end
  shardCount:SetText(shards)
end

function shardacecata:ScanInv()
	self.Timers.BagTimer = nil
	shardacecata:displayCurrentShardCount()

	local hsCount = 0
	for itemID,_ in pairs(self.HealthstoneID) do
		hsCount = hsCount + GetItemCount(itemID)
	end
	if hsCount == 0 then
		self.buttons[3]:Desaturate(true)
	else
		self.buttons[3]:Desaturate(false)
	end
--	local ssCount = 0
--	for itemID,_ in pairs(self.SoulstoneID) do
--		ssCount = ssCount + GetItemCount(itemID)
--	end
--	if ssCount == 0 then
--		self.buttons[2]:Desaturate(true)
	--else
--		self.buttons[2]:Desaturate(false)
--	end

end

function shardacecata:SetupTooltips()
	if self:getShowTooltips() then
		for _, button in ipairs({self.buttons[1]:GetChildren()}) do
			button:SetScript("OnEnter", function(self) GameTooltip_SetDefaultAnchor(GameTooltip, UIParent); GameTooltip:SetHyperlink(GetSpellLink(self:GetAttribute("spell"),"") or "spell:23161") end)
			button:SetScript("OnLeave", function(self) GameTooltip:Hide() end )
		end
		
		for i=2, 3, 1 do
			self.buttons[i]:SetScript("OnEnter", function(self) GameTooltip_SetDefaultAnchor(GameTooltip, UIParent); if self:GetAttribute("spell2") ~= nil then GameTooltip:SetHyperlink(GetSpellLink(self:GetAttribute("spell2"),"")) end end)
			self.buttons[i]:SetScript("OnLeave", function(self) GameTooltip:Hide() end )
    end

	else
		
		for _, button in ipairs({self.buttons[1]:GetChildren()}) do
			button:SetScript("OnEnter", nil)
			button:SetScript("OnLeave", nil)
		end
		
		for i=2, 3, 1 do
			self.buttons[i]:SetScript("OnEnter", nil)
			self.buttons[i]:SetScript("OnLeave", nil)
		end
	end
end


function shardacecata:CreateButtons()
	if InCombatLockdown() then
		CreateButtonsFlag = true
		return
	end
	
	local shardButton = BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", "Interface\\AddOns\\shardacecata\\shard", "SecureHandlerClickTemplate", UIParent)
	self.buttons = self.buttons or {
    shardButton,
    BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.CreateSS)), "SecureActionButtonTemplate", UIParent),
    BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.CreateHS)), "SecureActionButtonTemplate", UIParent),
  }
	self.firstrow = self.firstrow or {
    BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.SoulHarvest)), "SecureActionButtonTemplate", shardButton),
    BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.SoulLink)), "SecureActionButtonTemplate", shardButton),
    BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.Empower)), "SecureActionButtonTemplate", shardButton),
  }

	self.secondrow = self.secondrow or {
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.Soulburn)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.Imp)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.Voidwalker)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.Succubus)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.Felhunter)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.Felguard)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.Infernal)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.Doomguard)), "SecureActionButtonTemplate",shardButton),
	}
	self.thirdrow = self.thirdrow or {
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.FelArmor)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.DemonSkin)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.DarkIntent)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.UnendingBreath)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.ShadowWard)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.Metamorph)), "SecureActionButtonTemplate",shardButton),
	}
	self.forthrow = self.forthrow or {
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.Dreadsteed)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.Summon)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.SoulWell)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.Eye)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.Banish)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.Enslave)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.SummonCircle)), "SecureActionButtonTemplate",shardButton),
		BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", select(3,GetSpellInfo(self.spellID.PortCircle)), "SecureActionButtonTemplate",shardButton),
	}
	
	self.anchor = self.anchor or BD:NewButton("Interface\\AddOns\\shardacecata\\BtnRound", "Interface\\AddOns\\shardacecata\\blank", nil)
	self.anchor:Hide()
	 
	if strlower(self.db.profile.anchor or "") == "cursor" then
		self.anchor:SetPoint("CENTER",UIParent,"BOTTOMLEFT",self.db.profile.curx / UIParent:GetEffectiveScale(), self.db.profile.cury / UIParent:GetEffectiveScale() )
	else
		self.anchor:SetPoint("CENTER",UIParent,"BOTTOMLEFT",_G[self.db.profile.anchor or "Minimap"]:GetCenter() )
	end
end
function shardacecata:PositionButtons()
	if InCombatLockdown() then
		PositionButtonsFlag = true
		return
	end

	for _, button in pairs(self.buttons) do
		button:ClearAllPoints()
		button:SetPoint("BOTTOMRIGHT", UIParent, "TOPLEFT", 0,0)
		button:SetParent("UIParent")
		button:Show()
		button:Resize(26, 26)
	end

	for _, button in pairs(self.firstrow) do
		button:SetPoint("BOTTOMRIGHT", UIParent, "TOPLEFT", 0,0)
		button:SetParent(self.buttons[1])
		button:Resize(26, 26)
	end
	for _, button in pairs(self.secondrow) do
		button:SetPoint("BOTTOMRIGHT", UIParent, "TOPLEFT", 0,0)
		button:SetParent(self.buttons[1])
		button:Resize(26, 26)
	end
	for _, button in pairs(self.thirdrow) do
		button:SetPoint("BOTTOMRIGHT", UIParent, "TOPLEFT", 0,0)
		button:SetParent(self.buttons[1])
		button:Resize(26, 26)
	end
	for _, button in pairs(self.forthrow) do
		button:SetPoint("BOTTOMRIGHT", UIParent, "TOPLEFT", 0,0)
		button:SetParent(self.buttons[1])
		button:Resize(26, 26)
	end
	
	if self:getStraightState() then
		local x,y = BD:GetXY(self:getRadius(), self:getAngle())
		BD:ArrangeStraight(self.anchor, x, y, self:getMainOffX(), self:getMainOffY(), self.buttons)
	else
		BD:ArrangeCircle(self.anchor, self:getRadius(), self:getAngle(), self:getOverrideState() and self:getIncrement() or (((math.deg(math.acos(13/self:getRadius())))-90)*2)* self.db.profile.reverse, self.buttons)
	end
	BD:ArrangeStraight(self.buttons[1], self:get1InitX(), self:get1InitY(), self:get1OffX(), self:get1OffY(), self.firstmap)
	BD:ArrangeStraight(self.buttons[1], self:get2InitX(), self:get2InitY(), self:get2OffX(), self:get2OffY(), self.secondmap)
	BD:ArrangeStraight(self.buttons[1], self:get3InitX(), self:get3InitY(), self:get3OffX(), self:get3OffY(), self.thirdmap)
	BD:ArrangeStraight(self.buttons[1], self:get4InitX(), self:get4InitY(), self:get4OffX(), self:get4OffY(), self.forthmap)
end
function shardacecata:SetupButtons()
	if InCombatLockdown() then
		SetupButtonsFlag = true
		return
	end
	
	local tmpspell, mount =0,0

	for i = 1, GetNumCompanions("MOUNT") do
		_,_,spellNum,_,_ = GetCompanionInfo("MOUNT", i)
		if spellNum == self.spellID.Felsteed then
			mount = spellNum
		elseif spellNum == self.spellID.Dreadsteed then
			mount = spellNum
			break
		end
	end

	local hsmacro = ""
	local ssmacro = ""

	for id,_ in pairs(self.HealthstoneID) do
		hsmacro = format("/use item:%s\n%s", id, hsmacro)
	end
	for id,_ in pairs(self.SoulstoneID) do
		ssmacro = format("/use item:%s\n%s", id, ssmacro)
	end


	self.buttons[1]:RegisterForClicks("AnyUp")
	self.buttons[1]:SetAttribute("_onclick", [[
		
		if FlyOutButtonsOpen then
			for i, button in ipairs(MenuButtons) do
				button:Hide()
			end
			FlyOutButtonsOpen = false
		else
			
			for i, button in ipairs(MenuButtons) do
				button:Show()
			end
			FlyOutButtonsOpen = true
		end
	
	]])

	self.buttons[2]:SetAttribute("type2", "spell")
	self.buttons[2]:SetAttribute("spell2", GetSpellInfo(GetSpellInfo(self.spellID.CreateSS)))

	self.buttons[3]:SetAttribute("type2", "spell")
	self.buttons[3]:SetAttribute("spell2", GetSpellInfo(GetSpellInfo(self.spellID.CreateHS)))

	self.buttons[2]:SetAttribute("type1", "macro")
	self.buttons[2]:SetAttribute("macro1", ATTRIBUTE_NOOP)
	self.buttons[2]:SetAttribute("macrotext1", ssmacro)

	self.buttons[3]:SetAttribute("type1", "macro")
	self.buttons[3]:SetAttribute("macro1", ATTRIBUTE_NOOP)
	self.buttons[3]:SetAttribute("macrotext1", hsmacro)

	self.firstrow[1]:SetAttribute("type", "spell")
	self.firstrow[1]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.SoulHarvest)))
	self.firstrow[2]:SetAttribute("type", "spell")
	self.firstrow[2]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.SoulLink)))
	self.firstrow[3]:SetAttribute("type", "spell")
	self.firstrow[3]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Empower)))

	self.secondrow[1]:SetAttribute("type", "spell")
	self.secondrow[1]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Soulburn)))
	self.secondrow[2]:SetAttribute("type", "spell")
	self.secondrow[2]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Imp)))
	self.secondrow[3]:SetAttribute("type", "spell")
	self.secondrow[3]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Voidwalker)))
	self.secondrow[4]:SetAttribute("type", "spell")
	self.secondrow[4]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Succubus)))
	self.secondrow[5]:SetAttribute("type", "spell")
	self.secondrow[5]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Felhunter)))
	self.secondrow[6]:SetAttribute("type", "spell")
	self.secondrow[6]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Felguard)))
	self.secondrow[7]:SetAttribute("type", "spell")
	self.secondrow[7]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Infernal)))
	self.secondrow[8]:SetAttribute("type", "spell")
	self.secondrow[8]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Doomguard)))
	

	tmpspell = GetSpellInfo(GetSpellInfo(self.spellID.DemonArmor))
	if tmpspell == nil then
		tmpspell = GetSpellInfo(GetSpellInfo(self.spellID.DemonSkin))
	end
	self.thirdrow[1]:SetAttribute("type", "spell")
	self.thirdrow[1]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.FelArmor)))
	self.thirdrow[2]:SetAttribute("type", "spell")
    self.thirdrow[2]:SetAttribute("spell", tmpspell)
	self.thirdrow[3]:SetAttribute("type", "spell")
	self.thirdrow[3]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.DarkIntent)))
	self.thirdrow[4]:SetAttribute("type", "spell")
	self.thirdrow[4]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.UnendingBreath)))
	self.thirdrow[5]:SetAttribute("type", "spell")
	self.thirdrow[5]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.ShadowWard)))
	self.thirdrow[6]:SetAttribute("type", "spell")
	self.thirdrow[6]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Metamorph)))

	self.forthrow[1]:SetAttribute("type", "spell")
	self.forthrow[1]:SetAttribute("spell", GetSpellInfo(mount))
	self.forthrow[2]:SetAttribute("type", "spell")
	self.forthrow[2]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Summon)))
	self.forthrow[3]:SetAttribute("type", "spell")
	self.forthrow[3]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.SoulWell)))
	self.forthrow[4]:SetAttribute("type", "spell")
	self.forthrow[4]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Eye)))
	self.forthrow[5]:SetAttribute("type", "spell")
	self.forthrow[5]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Banish)))
	self.forthrow[6]:SetAttribute("type", "spell")
	self.forthrow[6]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Enslave)))
	self.forthrow[7]:SetAttribute("type", "spell")
	self.forthrow[7]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.SummonCircle)))
	self.forthrow[8]:SetAttribute("type", "spell")
	self.forthrow[8]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.PortCircle)))
	
	
	
	self.firstmap = {}
	self.secondmap = {}
	self.thirdmap = {}
	self.forthmap = {}



	for num, button in pairs(self.firstrow) do
		if button:GetAttribute("spell") then
			tinsert(self.firstmap, button)
		end
		--button:Hide()
	end
	for num, button in pairs(self.secondrow) do
		if button:GetAttribute("spell") then
			tinsert(self.secondmap, button)
		end
		--button:Hide()
	end
	for num, button in pairs(self.thirdrow) do
		if button:GetAttribute("spell") then
			tinsert(self.thirdmap, button)
		end
		--button:Hide()
	end
	for num, button in pairs(self.forthrow) do
		if button:GetAttribute("spell") then
			tinsert(self.forthmap, button)
		end
		--button:Hide()
	end
	self:SetupTooltips()
end

function shardacecata:Timer()
	if self.Timers.BagTimer then
		local timeleft = self.Timers.BagTimer - time()
		if (timeleft < 1) then
			self:ScanInv()
		end
	end
	if self.Timers.SoulTimer then
		local timeleft = self.Timers.SoulTimer - time()
		if(timeleft > 60)then
			soulTime:SetText((floor(timeleft / 60)) +1)
		else
			soulTime:SetText(timeleft)
		end
		soulTime:Show()
		
		if(timeleft < 1)then
			self.Timers.SoulTimer = nil
			soulTime:Hide()
			if self:getSoundState() then
				PlaySoundFile("Interface\\AddOns\\ShardAceCata\\SoulStone.ogg")
			end
			self.db.profile.SoulTime = nil
			DEFAULT_CHAT_FRAME:AddMessage("|cffffff00** Soulstone Cooldown over, ready to use **|r")
			UIErrorsFrame:AddMessage("** Soulstone Cooldown over, ready to use **", 0.92, 0.75, 0.05, 1.0, 12)
		end
	end

--Setup Desaturate for flyout buttons
	
	if self.Timers.HSTimer then
		local timeleft = self.Timers.HSTimer - time()
		if(timeleft > 60)then
			healthTime:SetText((floor(timeleft / 60)) +1)
		else
			healthTime:SetText(timeleft)
		end
		healthTime:Show()
		
		if(timeleft < 1)then
			self.Timers.HSTimer = nil
			self.db.profile.HSTimer = nil
			healthTime:Hide()
		end
	end
	if self.Timers.SoulHarvestTimer then
		local timeleft = self.Timers.SoulHarvestTimer - time()
		if timeleft >= 1 then
			self.firstrow[1]:Desaturate(true)
		else
			self.firstrow[1]:Desaturate(false)
			self.Timers.SoulHarvestTimer = nil
			self.db.profile.SoulHarvestTimer = nil
		end
	end
	if self.Timers.SoulburnTimer then
		local timeleft = self.Timers.SoulburnTimer - time()
		if timeleft >= 1 then
			self.secondrow[1]:Desaturate(true)
		else
			self.secondrow[1]:Desaturate(false)
			self.Timers.SoulburnTimer = nil
			self.db.profile.SoulburnTimer = nil
		end
	end
	if self.Timers.ShadowWardTimer then
		local timeleft = self.Timers.ShadowWardTimer - time()
		if timeleft >= 1 then
			self.thirdrow[5]:Desaturate(true)
		else
			self.thirdrow[5]:Desaturate(false)
			self.Timers.ShadowWardTimer = nil
			self.db.profile.ShadowWardTimer = nil
		end
	end
	if self.Timers.EmpowerTimer then
		local timeleft = self.Timers.EmpowerTimer - time()
		if timeleft >= 1 then
			self.firstrow[3]:Desaturate(true)
		else
			self.firstrow[3]:Desaturate(false)
			self.Timers.EmpowerTimer = nil
			self.db.profile.EmpowerTimer = nil
		end
	end
	if self.Timers.MetamorphTimer then
		local timeleft = self.Timers.MetamorphTimer - time()
		if timeleft >= 1 then
			self.thirdrow[6]:Desaturate(true)
		else
			self.thirdrow[6]:Desaturate(false)
			self.Timers.MetamorphTimer = nil
			self.db.profile.MetamorphTimer = nil
		end
	end
	if self.Timers.InfernalTimer then
		local timeleft = self.Timers.InfernalTimer - time()
		if timeleft >= 1 then
			self.secondrow[7]:Desaturate(true)
			self.secondrow[8]:Desaturate(true)
		else
			self.secondrow[7]:Desaturate(false)
			self.secondrow[8]:Desaturate(false)
			self.Timers.InfernalTimer = nil
			self.db.profile.InfernalTimer = nil
		end
	end
	if self.Timers.DoomguardTimer then
		local timeleft = self.Timers.DoomguardTimer - time()
		if timeleft >= 1 then
			self.secondrow[8]:Desaturate(true)
			self.secondrow[7]:Desaturate(true)
		else
			self.secondrow[8]:Desaturate(false)
			self.secondrow[7]:Desaturate(false)
			self.Timers.DoomguardTimer = nil
			self.db.profile.DoomguardTimer = nil
		end
	end
	if self.Timers.SummonTimer then
		local timeleft = self.Timers.SummonTimer - time()
		if timeleft >= 1 then
			self.forthrow[2]:Desaturate(true)
		else
			self.forthrow[2]:Desaturate(false)
			self.Timers.SummonTimer = nil
			self.db.profile.SummonTimer = nil
		end
	end
		if self.Timers.SoulWellTimer then
		local timeleft = self.Timers.SoulWellTimer - time()
		if timeleft >= 1 then
			self.forthrow[3]:Desaturate(true)
		else
			self.forthrow[3]:Desaturate(false)
			self.Timers.SoulWellTimer = nil
			self.db.profile.SoulWellTimer = nil
		end
	end
		if self.Timers.PortCircleTimer then
		local timeleft = self.Timers.PortCircleTimer - time()
		if timeleft >= 1 then
			self.forthrow[8]:Desaturate(true)
		else
			self.forthrow[8]:Desaturate(false)
			self.Timers.PortCircleTimer = nil
			self.db.profile.PortCircleTimer = nil
		end
	end

		
end

-- Event handlers

function shardacecata.UNIT_DISPLAYPOWER(self,event)
  shardacecata:displayCurrentShardCount()
end

function shardacecata.UNIT_POWER(self,event,arg1,arg2)
  if (arg1 == "player" ) then
    if ( arg2 == "SOUL_SHARDS" ) then
      shardacecata:displayCurrentShardCount()
    end
  end
end
function shardacecata:UNIT_SPELLCAST_SENT(event, unit, spell, rank, target)
	if spell == GetSpellInfo(6203) then -- Soulstone Resurrection
		SoulstoneTarget = target
	end
end

function shardacecata:UNIT_SPELLCAST_START(event, unit, spell, rank, target)
end

function shardacecata:UNIT_SPELLCAST_SUCCEEDED(event, unit, spell, rank, target)
	
	if unit == "player" then

		-- Set timers and make any announcements for that spell
	
		if spell == GetSpellInfo(6203) then
			self.Timers.SoulTimer = time() +600
			self.db.profile.SoulTime = self.Timers.SoulTimer
			self.timerFrame:Show()
			self:Timer()
			
		elseif spell == GetSpellInfo(5723) or spell == GetSpellInfo(5720) or spell == GetSpellInfo(6263) or spell == GetSpellInfo(11732) or spell == GetSpellInfo(27235) or spell == GetSpellInfo(6262) then
			self.Timers.HSTimer = time() + 60
			self.db.profile.HSTimer = self.Timers.HSTimer
			self.timerFrame:Show()
			self:Timer()
		
		elseif spell == GetSpellInfo(self.spellID.ShadowWard) then
			self.Timers.ShadowWardTimer = time() + 30
			self.db.profile.ShadowWardTimer = self.Timers.ShadowWardTimer
			self.timerFrame:Show()
			self:Timer()
		
		elseif spell == GetSpellInfo(self.spellID.Empower) then
			self.Timers.EmpowerTimer = time() + 60
			self.db.profile.EmpowerTimer = self.Timers.EmpowerTimer
			self.timerFrame:Show()
			self:Timer()
		
		elseif spell == GetSpellInfo(self.spellID.Metamorph) then
			self.Timers.MetamorphTimer = time() + 180
			self.db.profile.MetamorphTimer = self.Timers.MetamorphTimer
			self.timerFrame:Show()
			self:Timer()
		
		elseif spell == GetSpellInfo(self.spellID.Infernal) then
			self.Timers.InfernalTimer = time() + 600
			self.db.profile.InfernalTimer = self.Timers.InfernalTimer
			self.timerFrame:Show()
			self:Timer()
		
		elseif spell == GetSpellInfo(self.spellID.Doomguard) then
			self.Timers.DoomguardTimer = time() + 600
			self.db.profile.DoomguardTimer = self.Timers.DoomguardTimer
			self.timerFrame:Show()
			self:Timer()
		
		elseif spell == GetSpellInfo(self.spellID.SoulHarvest) then
			self.Timers.SoulHarvestTimer = time() + 30
			self.db.profile.SoulHarvestTimer = self.Timers.SoulHarvestTimer
			self.timerFrame:Show()
			self:Timer()
		
		elseif spell == GetSpellInfo(self.spellID.Soulburn) then
			self.Timers.SoulburnTimer = time() + 0
			self.db.profile.SoulburnTimer = self.Timers.SoulburnTimer
			self.timerFrame:Show()
			self:Timer()
	
		elseif spell == GetSpellInfo(self.spellID.PortCircle) then
			self.Timers.PortCircleTimer = time() + 30
			self.db.profile.PortCircleTimer = self.Timers.PortCircleTimer
			self.timerFrame:Show()
			self:Timer()
	
		-- Other announcements
		
		elseif spell == GetSpellInfo(self.spellID.SoulWell) then -- Ritual Of Souls
			self.Timers.SoulWellTimer = time() + 120
			self.db.profile.SoulWellTimer = self.Timers.SoulWellTimer
			self.timerFrame:Show()
			self:Timer()

		elseif spell == GetSpellInfo(self.spellID.Summon) then -- Ritual Of Summoning
			self.Timers.SummonTimer = time() + 0
			self.db.profile.SummonTimer = self.Timers.SummonTimer
			self.timerFrame:Show()
			self:Timer()		
		end
		
	end
	
end

--function shardacecata:BAG_UPDATE()
--	if not (self.Timers.BagTimer) then
--		self.Timers.BagTimer = time () + 1
--		self.timerFrame:Show()
--	end
--end

function shardacecata:PLAYER_REGEN_ENABLED()
	if CreateButtonsFlag then
		self:CreateButtons()
		CreateButtonsFlag = nil
	end
	if SetupButtonsFlag then
		self:SetupButtons()
		SetupButtonsFlag = nil
	end
	if PositionButtonsFlag then
		self:PositionButtons()
		PositionButtonsFlag = nil
	end
	
end

function shardacecata:SPELLS_CHANGED()
	self:SetupButtons()
	self:PositionButtons()
end

-- Initialization

function shardacecata:OnInitialize()
	self.db = AceDB:New("ShardAceCataDB",{
		profile = {
			radius = 105,
			angle = 270,
			reverse = 1,
			increment = 15,
			mainoffx = 28,
			mainoffy = 0,
			oneinitx = -28,
			oneinity = 28,
			oneoffx = -28,
			oneoffy = 0,
			twoinitx = -28,
			twoinity = 0,
			twooffx = -28,
			twooffy = 0,
			threeinitx = -28,
			threeinity = -28,
			threeoffx = -28,
			threeoffy = 0,
			fourinitx = -28,
			fourinity = -56,
			fouroffx = -28,
			fouroffy = 0,
			reversesort = 0,
			soulstonemessage = 'Soulstone cast on %t.',
			enablesoulstonemessage = true,
			ritualofsoulsmessage = 'Please click for cookies of green goodness.',
			enableritualofsoulsmessage = true,
			ritualofsummoningmessage = 'Please click for demonic taxi stone of evil.',
			enableritualofsummoningmessage = true,
			showtooltips = true,
			soundstate = true,
			saymessages = false,
		}
	},UnitName("player").." - "..GetRealmName())
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	AceConfig:RegisterOptionsTable("shardacecata", options)
    --LibStub("AceConfig-3.0"):RegisterOptionsTable("Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db))
	if self.db:GetCurrentProfile() == "profile" then
		self.db:SetProfile(UnitName("Player") .. " - " .. GetRealmName())
		self.db:CopyProfile("profile")
		self.db:DeleteProfile("profile")
	end



	dialog:AddToBlizOptions("shardacecata", "ShardAce Cataclysm")
end

function shardacecata:OnEnable()
	self:CreateButtons()
	self:SPELLS_CHANGED()
	
	self.buttons[1]:Execute( [[MenuButtons = newtable(self:GetChildren())]])
	for _, button in ipairs({self.buttons[1]:GetChildren()}) do
		self.buttons[1]:WrapScript(button, "OnClick",
			[[
			if button == "LeftButton" then
				return button, "hide"
			else
				return button, nil
			end
			]],
			[[for i, button in ipairs(MenuButtons) do
				button:Hide()
			FlyOutButtonsOpen = false
			
			end]]
		)
	end
	
	self:RegisterEvent("LEARNED_SPELL_IN_TAB", "SPELLS_CHANGED")
	self:RegisterEvent("SPELLS_CHANGED", "SPELLS_CHANGED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	--self:RegisterEvent("BAG_UPDATE")
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	--self:RegisterEvent("UNIT_POWER")
  	self:RegisterEvent("UNIT_DISPLAYPOWER");
		
	soulTime = self.buttons[2]:CreateFontString("DDSoulTime", "OVERLAY")
	soulTime:SetFontObject(GameFontHighlightSmall)
	soulTime:SetTextColor(1,1,1,1)
	soulTime:SetAllPoints()
	healthTime = self.buttons[3]:CreateFontString("DDHealthTime", "OVERLAY")
	healthTime:SetFontObject(GameFontHighlightSmall)
	healthTime:SetTextColor(1,1,1,1)
	healthTime:SetAllPoints()
	
	self.timerFrame = CreateFrame("Frame","DDTimerFrame")
	self.timerFrame:Hide()
	self.Timers = {}
	self.timerFrame:SetScript("OnUpdate", function(self, arg1) shardacecata.Timer(shardacecata, arg1) end)
	
	
	local timems = time()
	if self.db.profile.SoulHarvestTimer or 0 > timems then
		self.Timers.SoulHarvestTimer = self.db.profile.SoulHarvestTimer
		self.timerFrame:Show()
		self:Timer()
	end
	if self.db.profile.SoulburnTimer or 0 > timems then
		self.Timers.SoulburnTimer = self.db.profile.SoulburnTimer
		self.timerFrame:Show()
		self:Timer()
	end
	if self.db.profile.SoulTime or 0 > timems then
		self.Timers.SoulTimer = self.db.profile.SoulTime
		self.timerFrame:Show()
		self:Timer()
	end
	if self.db.profile.HSTimer or 0 > timems then
		self.Timers.HSTimer = self.db.profile.HSTimer
		self.timerFrame:Show()
		self:Timer()
	end
	if self.db.profile.ShadowWardTimer or 0 > timems then
		self.Timers.ShadowWardTimer = self.db.profile.ShadowWardTimer
		self.timerFrame:Show()
		self:Timer()
	end
	if self.db.profile.EmpowerTimer or 0 > timems then
		self.Timers.EmpowerTimer = self.db.profile.EmpowerTimer
		self.timerFrame:Show()
		self:Timer()
	end
	if self.db.profile.MetamorphTimer or 0 > timems then
		self.Timers.MetamorphTimer = self.db.profile.MetamorphTimer
		self.timerFrame:Show()
		self:Timer()
	end
	if self.db.profile.InfernalTimer or 0 > timems then
		self.Timers.InfernalTimer = self.db.profile.InfernalTimer
		self.timerFrame:Show()
		self:Timer()
	end
	if self.db.profile.DoomguardTimer or 0 > timems then
		self.Timers.DoomguardTimer = self.db.profile.DoomguardTimer
		self.timerFrame:Show()
		self:Timer()
	end
	if self.db.profile.PortCricleTimer or 0 > timems then
		self.Timers.PortCircleTimer = self.db.profile.PortCircleTimer
		self.timerFrame:Show()
		self:Timer()
	end
		if self.db.profile.SummonTimer or 0 > timems then
		self.Timers.SummonTimer = self.db.profile.SummonTimer
		self.timerFrame:Show()
		self:Timer()
	end
		if self.db.profile.SoulWellTimer or 0 > timems then
		self.Timers.SoulWellTimer = self.db.profile.SoulWellTimer
		self.timerFrame:Show()
		self:Timer()
	end
	
	lastSort = GetTime()

end
